// ignore_for_file: unused_local_variable

import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_auth_helper/src/auth_error.dart';
import 'package:firebase_auth_helper/src/auth_provider.dart';

import './firebase_error_codes.dart';
import '../auth_user.dart';
import '../credentials.dart';

//interesting: https://github.com/synapsecode/fireauth/blob/master/lib/fireauth.dart

/// see https://firebase.flutter.dev/docs/auth/usage/
class FirebaseAuthProvider implements AuthProvider {
  final FirebaseAuth firebaseAuth;

  const FirebaseAuthProvider(this.firebaseAuth);

  factory FirebaseAuthProvider.fromInstance() {
    return FirebaseAuthProvider(FirebaseAuth.instance);
  }

  AuthUser? _toAuthUser(User? user) {
    if (user == null) return null;
    return AuthUser(user.uid,
        displayName: user.displayName ?? "",
        email: user.email ?? "",
        emailVerified: user.emailVerified,
        isAnonymous: user.isAnonymous,
        metadata: user.metadata,
        phoneNumber: user.phoneNumber,
        photoURL: user.photoURL,
        providerData: user.providerData,
        refreshToken: user.refreshToken,
        updateDisplayName: user.updateDisplayName,
        sendVerificationEmail: user.sendEmailVerification,
        reload: user.reload,
        getIdToken: user.getIdToken);
  }

  @override
  Stream<AuthUser?> get authStateChanged =>
      firebaseAuth.authStateChanges().map(_toAuthUser);

  @override
  AuthUser? get currentAuthUser => _toAuthUser(firebaseAuth.currentUser);

  @override
  Stream<AuthUser?> get userChanges =>
      firebaseAuth.userChanges().map(_toAuthUser);

  @override
  Future<AuthUser?> updateDisplayName(String? name) async {
    final user = firebaseAuth.currentUser;
    if (user == null) return Future.value(null);
    await user.updateDisplayName(name);
    return _toAuthUser(firebaseAuth.currentUser);
  }

  @override
  Future<String?> emailPasswordSignIn(LoginCredentials credentials) async {
    //see https://firebase.flutter.dev/docs/auth/usage/#emailpassword-registration--sign-in
    try {
      UserCredential userCredential = await FirebaseAuth.instance
          .signInWithEmailAndPassword(
              email: credentials.email, password: credentials.password);
      return null;
    } on FirebaseAuthException catch (e) {
      if (e.code.contains(FirebaseAuthErrorCode.wrongPassword)) {
        return "Wrong password";
      }
      if (e.code == 'weak-password') {
        return 'The password provided is too weak.';
      } else if (e.code == 'email-already-in-use') {
        return 'The account already exists for that email.';
      } else {
        return ("Failed to login");
      }
    } catch (e) {
      return "Failed to login $e";
    }
  }

  @override
  Future<void> signOut() async {
    await FirebaseAuth.instance.signOut();
  }

  /// Throws an [AuthError]
  @override
  Future? deleteCurrentUser() async {
    try {
      await firebaseAuth.currentUser?.delete();
    } on FirebaseAuthException catch (err) {
      throw AuthError(code: err.code);
    }
  }
}
