import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_auth_helper/firebase_auth_helper.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/annotations.dart';
import 'package:mockito/mockito.dart';

import 'auth_state_wrapper_test.mocks.dart';

@GenerateMocks([FirebaseAuthProvider, UserMetadata, UserInfo])
main() {
  late MockFirebaseAuthProvider authProvider;

  setUp(() {
    authProvider = MockFirebaseAuthProvider();
  });

  Future<void> pumpWrapper(
    WidgetTester tester, {
    Widget? signedIn,
    Widget? notSignedIn,
    Widget? loading,
    Widget Function(BuildContext context, Object? error)? errorBuilder,
  }) {
    signedIn ??= const Text("Signed in as : ");
    notSignedIn ??= Column(
      children: [
        const Text("Logged out"),
        TextButton(child: const Text("Login"), onPressed: () {}),
      ],
    );
    loading ??= const Text("Loading");

    const errorWidget = Text("Error");

    return tester.pumpWidget(MaterialApp(
      home: Material(
        child: Scaffold(
            body: AuthStateWrapper(
                authProvider: authProvider,
                signedIn: (c) => signedIn!,
                notSignedIn: (c) => notSignedIn!,
                loading: (c) => loading!,
                error: errorBuilder ?? (c, e) => errorWidget)),
      ),
    ));
  }

  testWidgets("Displays loading if no value yet", (WidgetTester tester) async {
    when(authProvider.authStateChanged).thenAnswer((_) => Stream.value(null));
    await pumpWrapper(tester);
    // no pumpAndSettle() !!;

    expect(find.text("Loading"), findsOneWidget);
  });

  testWidgets("Displays error if error occurred", (WidgetTester tester) async {
    when(authProvider.authStateChanged)
        .thenAnswer((_) => Stream.error(Error()));

    await pumpWrapper(tester);
    await tester.pumpAndSettle();

    expect(find.text("Error"), findsOneWidget);
  });

  testWidgets("Displays not signed in if user is null",
      (WidgetTester tester) async {
    when(authProvider.authStateChanged).thenAnswer((_) => Stream.value(null));

    const notSignedIn = Text("Logged out");
    await pumpWrapper(tester, notSignedIn: notSignedIn);
    await tester.pumpAndSettle();

    expect(find.byWidget(notSignedIn), findsOneWidget);
  });

  testWidgets("Displays signed in if user is not null",
      (WidgetTester tester) async {
    final authUser = AuthUser("user123",
        displayName: "John",
        email: "some@email.com",
        emailVerified: false,
        isAnonymous: false,
        metadata: MockUserMetadata(),
        phoneNumber: null,
        photoURL: "https://someUrl",
        providerData: const [],
        refreshToken: null,
        updateDisplayName: (s) => Future.delayed(Duration.zero),
        sendVerificationEmail: () => Future.delayed(Duration.zero),
        reload: () => Future.delayed(Duration.zero),
        getIdToken: ([bool b = false]) => Future.value("someToken"));

    when(authProvider.authStateChanged)
        .thenAnswer((_) => Stream.value(authUser));

    final signedIn = Text("Logged in as ${authUser.email}");
    await pumpWrapper(tester, signedIn: signedIn);
    await tester.pumpAndSettle();

    expect(find.byWidget(signedIn), findsOneWidget);
  });
}
